package org.apache.log4j;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URL;
import java.security.CodeSource;
import java.util.HashMap;
import java.util.Map;
import org.apache.log4j.DefaultThrowableRenderer;
import org.apache.log4j.spi.ThrowableRenderer;

public final class EnhancedThrowableRenderer implements ThrowableRenderer {

   private Method getStackTraceMethod;
   private Method getClassNameMethod;


   public EnhancedThrowableRenderer() {
      try {
         Object ex = null;
         this.getStackTraceMethod = Throwable.class.getMethod("getStackTrace", (Class[])ex);
         Class ste = Class.forName("java.lang.StackTraceElement");
         this.getClassNameMethod = ste.getMethod("getClassName", (Class[])ex);
      } catch (Exception var3) {
         ;
      }

   }

   public String[] doRender(Throwable throwable) {
      if(this.getStackTraceMethod != null) {
         try {
            Object ex = null;
            Object[] elements = (Object[])((Object[])this.getStackTraceMethod.invoke(throwable, (Object[])ex));
            String[] lines = new String[elements.length + 1];
            lines[0] = throwable.toString();
            HashMap classMap = new HashMap();

            for(int i = 0; i < elements.length; ++i) {
               lines[i + 1] = this.formatElement(elements[i], classMap);
            }

            return lines;
         } catch (Exception var7) {
            ;
         }
      }

      return DefaultThrowableRenderer.render(throwable);
   }

   private String formatElement(Object element, Map classMap) {
      StringBuffer buf = new StringBuffer("\tat ");
      buf.append(element);

      try {
         String ex = this.getClassNameMethod.invoke(element, (Object[])null).toString();
         Object classDetails = classMap.get(ex);
         if(classDetails != null) {
            buf.append(classDetails);
         } else {
            Class cls = this.findClass(ex);
            int detailStart = buf.length();
            buf.append('[');

            try {
               CodeSource pkg = cls.getProtectionDomain().getCodeSource();
               if(pkg != null) {
                  URL implVersion = pkg.getLocation();
                  if(implVersion != null) {
                     if("file".equals(implVersion.getProtocol())) {
                        String path = implVersion.getPath();
                        if(path != null) {
                           int lastSlash = path.lastIndexOf(47);
                           int lastBack = path.lastIndexOf(File.separatorChar);
                           if(lastBack > lastSlash) {
                              lastSlash = lastBack;
                           }

                           if(lastSlash > 0 && lastSlash != path.length() - 1) {
                              buf.append(path.substring(lastSlash + 1));
                           } else {
                              buf.append(implVersion);
                           }
                        }
                     } else {
                        buf.append(implVersion);
                     }
                  }
               }
            } catch (SecurityException var13) {
               ;
            }

            buf.append(':');
            Package pkg1 = cls.getPackage();
            if(pkg1 != null) {
               String implVersion1 = pkg1.getImplementationVersion();
               if(implVersion1 != null) {
                  buf.append(implVersion1);
               }
            }

            buf.append(']');
            classMap.put(ex, buf.substring(detailStart));
         }
      } catch (Exception var14) {
         ;
      }

      return buf.toString();
   }

   private Class findClass(String className) throws ClassNotFoundException {
      try {
         return Thread.currentThread().getContextClassLoader().loadClass(className);
      } catch (ClassNotFoundException var5) {
         try {
            return Class.forName(className);
         } catch (ClassNotFoundException var4) {
            return this.getClass().getClassLoader().loadClass(className);
         }
      }
   }
}
